# -*- coding: utf-8 -*-
# setup.py
import sys

from setuptools import setup, find_packages

from sfftkrw.conf import SFFTKRW_VERSION, SFFTKRW_ENTRY_POINT

with open(u'README.rst') as f:
    long_description = f.read()

SFFTKRW_NAME = u"sfftk-rw"
SFFTKRW_AUTHOR = u"Paul K. Korir, PhD"
SFFTKRW_AUTHOR_EMAIL = u"pkorir@ebi.ac.uk, paul.korir@gmail.com"
SFFTKRW_DESCRIPTION = u"Toolkit for reading and writing EMDB-SFF files"
SFFTKRW_DESCRIPTION_CONTENT_TYPE = u'text/x-rst; charset=UTF-8'
SFFTKRW_URL = u"http://sfftk-rw.readthedocs.io/en/latest/index.html"
SFFTKRW_LICENSE = u"Apache License 2.0"
SFFTKRW_KEYWORDS = [u"EMDB-SFF", u"SFF", u"segmentation"]
SFFTKRW_INSTALL_REQUIRES = ['six', 'numpy', "lxml", 'h5py', 'RandomWords']
SFFTKRW_CLASSIFIERS = [
    # maturity
    u"Development Status :: 4 - Beta",
    # environment
    u"Environment :: Console",
    u"Intended Audience :: Developers",
    u"Intended Audience :: Science/Research",
    # license
    u"License :: OSI Approved :: Apache Software License",
    # os
    u"Operating System :: OS Independent",
    # python versions
    u"Programming Language :: Python :: 3",
    u"Programming Language :: Python :: 3.6",
    u"Programming Language :: Python :: 3.7",
    u"Programming Language :: Python :: 3.8",
    u"Programming Language :: Python :: 3.9",
    u"Programming Language :: Python :: 3.10",
    u"Programming Language :: Python :: 3.11",
    u"Topic :: Software Development :: Libraries :: Python Modules",
    u"Topic :: Terminals",
    u"Topic :: Text Processing",
    u"Topic :: Text Processing :: Markup",
    u"Topic :: Utilities",
]
setup(
    name=SFFTKRW_NAME,
    version=SFFTKRW_VERSION,
    packages=find_packages(),
    author=SFFTKRW_AUTHOR,
    author_email=SFFTKRW_AUTHOR_EMAIL,
    description=SFFTKRW_DESCRIPTION,
    long_description=long_description,
    long_description_content_type=SFFTKRW_DESCRIPTION_CONTENT_TYPE,
    url=SFFTKRW_URL,
    license=SFFTKRW_LICENSE,
    keywords=SFFTKRW_KEYWORDS,
    install_requires=SFFTKRW_INSTALL_REQUIRES,
    classifiers=SFFTKRW_CLASSIFIERS,
    python_requires=">=3.6",
    entry_points={
        'console_scripts': [
            '{} = sfftkrw.sffrw:main'.format(SFFTKRW_ENTRY_POINT),
        ]
    },
)



# -*- coding: utf-8 -*-
"""

Convenience utilities

"""
from __future__ import print_function

import sys
from functools import partial

_print = partial(print, file=sys.stderr)

# redefinitions used globally
if sys.version_info[0] > 2:
    import builtins

    # xrange
    _xrange = builtins.range
    # dictionaries preserve order in Python3.7+
    if sys.version_info[1] >= 7:
        _dict = builtins.dict
        _classic_dict = builtins.dict
    else:
        from collections import OrderedDict

        _dict = OrderedDict
        _classic_dict = builtins.dict

    # iter* methods on dictionaries
    _dict_iter_keys = _dict.keys
    _dict_iter_values = _dict.values
    _dict_iter_items = _dict.items

    # UserList
    from collections import UserList

    _UserList = UserList
    # pseudo unicode function (does not exist in Python3)
    _bytes = builtins.bytes
    _str = builtins.str


    def _unicode(data):
        return data


    # urlencode
    from urllib.parse import urlencode

    _urlencode = urlencode
    # string base is str
    _basestring = (builtins.bytes, builtins.str)


    # decode is meaningless for str in Python3
    def _decode(data, encoding):
        if isinstance(data, builtins.str):
            return data
        elif isinstance(data, builtins.bytes):
            return data.decode(encoding)
        return data


    # encoding
    def _encode(data, encoding):
        if isinstance(data, builtins.str):
            return data.encode(encoding)
        elif isinstance(data, builtins.bytes):
            return data
        return data


    # input
    _input = builtins.input

    # file
    import io

    _file = io.IOBase


    # clear list
    def _clear(_list):
        _list.clear()


    import inspect

    _getattr_static = inspect.getattr_static

    # exceptions
    _FileNotFoundError = FileNotFoundError
else:
    import __builtin__

    # xrange
    _xrange = __builtin__.xrange

    # for order preservation in dicts user OrderedDict
    from collections import OrderedDict

    _dict = OrderedDict
    _classic_dict = __builtin__.dict

    # iter* methods on dictionaries
    _dict_iter_keys = _dict.iterkeys
    _dict_iter_values = _dict.itervalues
    _dict_iter_items = _dict.iteritems

    from UserList import UserList

    _UserList = UserList
    # unicode
    _bytes = __builtin__.str
    _str = __builtin__.unicode


    def _unicode(data):
        return __builtin__.unicode(data)


    # urlencode
    from urllib import urlencode

    _urlencode = urlencode
    # string base is basestring
    _basestring = __builtin__.basestring


    # decode Python2 str object
    def _decode(data, encoding):
        if isinstance(data, __builtin__.str):
            return data.decode(encoding)
        elif isinstance(data, __builtin__.unicode):
            return data
        return data


    # encoding
    def _encode(data, encoding):
        if isinstance(data, __builtin__.str):
            return data
        elif isinstance(data, __builtin__.unicode):
            return data.encode(encoding)
        return data


    # input
    _input = __builtin__.raw_input

    # file
    _file = __builtin__.file


    # clear list
    def _clear(_list):
        del _list[:]


    # borrowed from Python 3.8's inspect.py
    # ------------------------------------------------ static version of getattr
    import types

    _sentinel = object()


    def _static_getmro(klass):
        return type.__dict__['__mro__'].__get__(klass)


    def _check_instance(obj, attr):
        instance_dict = {}
        try:
            instance_dict = object.__getattribute__(obj, "__dict__")
        except AttributeError:
            pass
        return dict.get(instance_dict, attr, _sentinel)


    def _check_class(klass, attr):
        for entry in _static_getmro(klass):
            if _shadowed_dict(type(entry)) is _sentinel:
                try:
                    return entry.__dict__[attr]
                except KeyError:
                    pass
        return _sentinel


    def _is_type(obj):
        try:
            _static_getmro(obj)
        except TypeError:
            return False
        return True


    def _shadowed_dict(klass):
        dict_attr = type.__dict__["__dict__"]
        for entry in _static_getmro(klass):
            try:
                class_dict = dict_attr.__get__(entry)["__dict__"]
            except KeyError:
                pass
            else:
                if not (type(class_dict) is types.GetSetDescriptorType and
                        class_dict.__name__ == "__dict__" and
                        class_dict.__objclass__ is entry):
                    return class_dict
        return _sentinel


    def getattr_static(obj, attr, default=_sentinel):
        """Retrieve attributes without triggering dynamic lookup via the
           descriptor protocol,  __getattr__ or __getattribute__.
           Note: this function may not be able to retrieve all attributes
           that getattr can fetch (like dynamically created attributes)
           and may find attributes that getattr can't (like descriptors
           that raise AttributeError). It can also return descriptor objects
           instead of instance members in some cases. See the
           documentation for details.
        """
        instance_result = _sentinel
        if not _is_type(obj):
            klass = type(obj)
            dict_attr = _shadowed_dict(klass)
            if (dict_attr is _sentinel or
                    type(dict_attr) is types.MemberDescriptorType):
                instance_result = _check_instance(obj, attr)
        else:
            klass = obj

        klass_result = _check_class(klass, attr)

        if instance_result is not _sentinel and klass_result is not _sentinel:
            if (_check_class(type(klass_result), '__get__') is not _sentinel and
                    _check_class(type(klass_result), '__set__') is not _sentinel):
                return klass_result

        if instance_result is not _sentinel:
            return instance_result
        if klass_result is not _sentinel:
            return klass_result

        if obj is klass:
            # for types we check the metaclass too
            for entry in _static_getmro(type(klass)):
                if _shadowed_dict(type(entry)) is _sentinel:
                    try:
                        return entry.__dict__[attr]
                    except KeyError:
                        pass
        if default is not _sentinel:
            return default
        raise AttributeError(attr)


    _getattr_static = getattr_static

    # exceptions
    _FileNotFoundError = OSError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
print_tools.py
==============

Utilities for printing stuff to the screen.
"""
from __future__ import division, print_function

import string
import sys
import time

from ..core import _basestring, _bytes, _decode, _encode

__author__ = 'Paul K. Korir, PhD'
__email__ = 'pkorir@ebi.ac.uk'
__date__ = '2016-16-06'
__updated__ = '2018-02-14'


def print_date(print_string, stream=sys.stderr, newline=True, incl_date=True):
    """Print the given string with date on the given stream
    
    :param print_string: the bytes/str to be printed
    :type print_string: bytes or str or unicode
    :param stream: the stream/file to write to
    :type stream: :py:obj:`sys.stderr` or :py:obj:`sys.stdout`
    :param bool newline: whether (default) or not to add a newline at the end
    :param bool incl_date: whether (default) or not to include the date at the beginning
    """
    try:
        assert isinstance(print_string, _basestring)
    except AssertionError:
        raise ValueError(u"input should be subclass of basestring: str or unicode")
    if isinstance(print_string, _bytes):
        print_string = _decode(print_string, 'utf-8')
    if newline:
        if incl_date:
            print(u"%s\t%s" % (time.ctime(time.time()), print_string), file=stream)
        else:
            print(u"%s" % (print_string), file=stream)
    else:
        if incl_date:
            print(u"%s\t%s" % (time.ctime(time.time()), print_string), file=stream, end='')
        else:
            print(u"%s" % (print_string), file=stream, end='')


def get_printable_ascii_string(B):
    """Given a bytes string of ASCII and non-ASCII return the maximal printable prefix
    
    :param bytes B: the bytes string to search
    :return ascii_b: the maximal ASCII prefix
    :rtype: bytes
    """
    B = _encode(B, u'utf-8')
    printables = list(map(ord, string.printable))
    index = len(B) - 1
    if sys.version_info[0] > 2:
        for i, b in enumerate(B): # for each byte
            if b not in printables:
                index = i
                break
    else:
        for i, b in enumerate(B): # for each byte
            _b = ord(b)
            if _b not in printables:
                index = i
                break
    ascii_b = B[:index]
    return ascii_b


def print_static(print_string, stream=sys.stderr, incl_date=True):
    """Print the given string with date on the given stream from the first position 
    overwriting any characters. 
    
    This is a useful way to display progress without overcrowding the screen.
    
    :param print_string: the bytes/str to be printed
    :type print_string: bytes or str
    :param stream: the stream/file to write to
    :type stream: :py:obj:`sys.stderr` or :py:obj:`sys.stdout`
    :param bool incl_date: whether (default) or not to include the date at the beginning
    """
    try:
        assert isinstance(print_string, _basestring)
    except AssertionError:
        raise ValueError(u"input should be subclass of basestring: str or unicode")
    print_string = _decode(print_string, u'utf-8')
    if incl_date:
        print(u"\r{}\t{}".format(time.ctime(time.time()), print_string), file=stream, end='')
    else:
        print(u"\r{}".format(print_string), file=stream, end='')
